# frozen_string_literal: true

require 'ice_nine'
require 'ice_nine/core_ext/object'

module Yousty
  module HTTP
    # A value object ensuring that the HTTP method provided is valid
    #
    class RequestMethod
      InvalidMethodError = Class.new(StandardError)

      SUPPORTED_METHODS = %w[get post patch delete].freeze

      # Compares other value with Symbol, String or other RequestMethod
      # @param [RequestMethod|String|Symbol]
      # @return [Boolean]
      #
      def ==(other)
        name == other.to_s.downcase
      end

      # Returns a unified string representation of the HTTP method
      # @return [String]
      #
      def to_s
        name
      end

      # Initializes deeply frozen RequestMethod object
      # @param [Symbol] a name of the HTTP method to be initialized
      #
      def self.new(*args, &block)
        super.deep_freeze
      end

      private

      attr_reader :name

      def initialize(name)
        parsed_name = name.to_s.downcase
        raise InvalidMethodError unless SUPPORTED_METHODS.include?(parsed_name)

        @name = parsed_name
      end
    end
  end
end
